PGFileDownloader = function (manager, serverDirectory, filename, filesize, localDirectory, jobId){

var _cbPage = 10240;
var _cbComplete = 0;
var _err;

function init()
{
    downloadChunk();
};

function downloadChunk()
{
    var sftpreq = {};
    sftpreq.type = "rf";
    sftpreq.path = serverDirectory + filename;
    sftpreq.offset = _cbComplete;
    sftpreq.count = Math.min(_cbPage, filesize-_cbComplete);
    var eof = (filesize-_cbComplete <= _cbPage);
    sftpreq.cache = !eof;
    sftpreq.localpath = localDirectory + filename; // native code should write file directly
    manager.sendRequest(jobId, sftpreq);
};

this.handleResponse = function(rsp)
{
    if (rsp.status === 0) {
        _cbComplete += parseInt(rsp.data); // data is the size of last downloaded chunk
        if (_cbComplete < filesize) {
            downloadChunk();
        }
        else {
            // complete!!!
            manager.onDownloadJobCompleted(jobId);
        }
    } else {
        _err = rsp.data;
        manager.onJobFailed(jobId, _err);
    }
};

this.getProgress = function()
{
    var progress;
    if (_err == undefined) {
        progress = Math.round(_cbComplete / filesize * 100) + "%";
    }
    else {
        progress = _err;
    }
    return [
                jobId,
                "D",
                filename,
                progress,
                filesize,
                _cbComplete
           ];
};

this.cancel = function()
{
    _err = "Cancelled";
    manager.onJobFailed(jobId, _err);
};

function onFileError(e)
{
    _err = "File error";
    manager.onJobFailed(jobId, _err);
};

init();
};

PGFileUploader = function (manager, directory, file, jobId){

var _filesize = 0;
var _cbPage = 10240;
var _cbComplete = 0;
var _err;

function init()
{
    _filesize = file.size;
    uploadChunk();
};

function uploadChunk()
{
    var sftpreq = {};
    sftpreq.type = "wf";
    sftpreq.path = directory+file.name;
    sftpreq.offset = _cbComplete;
    sftpreq.cache = (_filesize-_cbComplete > _cbPage);
    sftpreq.pagesize = _cbPage;
    sftpreq.localpath = file.path;
    manager.sendRequest(jobId, sftpreq);
};

this.handleResponse = function(rsp)
{
    if (rsp.status === 0) {
        _cbComplete += rsp.data;
        if (_cbComplete < _filesize) {
            uploadChunk();
        }
        else {
            // complete!!!
            manager.onUploadJobCompleted(jobId, directory);
        }
    }
    else {
        _err = rsp.data;
        manager.onJobFailed(jobId, _err);
    }
};

this.getProgress = function()
{
    var progress;
    if (_err == undefined) {
        progress = Math.round(_cbComplete / _filesize * 100) + "%"
    }
    else {
        progress = _err;
    }
    return [
                jobId,
                "U",
                file.name,
                progress,
                _filesize,
                _cbComplete
           ];
};

this.cancel = function()
{
    _err = "Cancelled";
    manager.onJobFailed(jobId, _err);
}

init();
};

PGLocalDirectoryView = function (gridControl, pathControl, session, sessionInfo, onUpdateState){

var _that = this;
var _path = (window.NRPhoneGapHomeDirectory != null) ? window.NRPhoneGapHomeDirectory : '/';
var _newpath;

function init()
{
    session.setEventHandler('local', handleResponse);
    _that.listDirectory();
};

this.listDirectory = function(path)
{
    if (path == null) {
        path = _path; // refresh current _path
    }
    _newpath = SFTP.Utility.canonicalizePath(path);
    var sftpreq = {};
    sftpreq.type = "lls";
    sftpreq.path = _newpath;
    sftpreq.unhide = sessionInfo.unhide;
    session.sendRequest(sftpreq);

    onUpdateState('commandlocal', 'lls');
};

this.upDirectory = function()
{
    if ("/" === _path) return;
    
    var temppath = _path.slice(0, _path.length-1);
    temppath = temppath.slice(0, temppath.lastIndexOf('/')+1);
    this.listDirectory(temppath);
};

this.getCurrentDirectory = function()
{
    return _path;
};

this.getSelectedFile = function()
{
    var selection = $(gridControl).flexReturnSelected();
    if (selection.length === 0) return null; // no selection
    if (selection[4] /*type*/ !== "1" /*file*/) return null;
    var file = {};
    file.name = selection[1] /*itemname*/;
    file.path = _path + "/" + selection[1] /*itemname*/;
    file.size = parseInt(selection[2].replace(/,/gi,'')) /*size*/;
    return file;
};

this.openSelectedItem = function()
{
    var selection = $(gridControl).flexReturnSelected();
    if (selection.length === 0) return; // no selection
    if (selection[1] === "..") {
        this.upDirectory();
        return;
    }
    if (selection[4] /*type*/ === "1" /*file*/) {
        var req = {};
        req.type = "OpenFile";
        req.filepath = _path + "/" + selection[1] /*itemname*/;
        NRClientOM.send(JSON.stringify(req));
    } else {
        this.listDirectory(_path + selection[1]);
    }
};

this.deleteSelectedItem = function()
{
    var selection = $(gridControl).flexReturnSelected();
    if (selection.length === 0) return; // no selection

    Ext.Msg.confirm('Confirm delete',"Are you sure you want to delete '" + selection[1] + "'?", function(e) {
        if(e == 'yes') {
            var sftpreq = {};
            if (selection[4] /*type*/ === "2") {
                sftpreq.type = "lrd"; // directory
            } else {
                sftpreq.type = "ldf"; // file
            }
            sftpreq.path = _path + "/" + selection[1] /*itemname*/;
            session.sendRequest(sftpreq);

            document.body.style.cursor = 'wait';
            onUpdateState('commandlocal', sftpreq.type);
        }
    });
};

this.refreshDirectory = function()
{
    this.listDirectory();
};

this.moveSeletedItem = function()
{
    var selection = $(gridControl).flexReturnSelected();
    if (selection.length === 0) return; // no selection

    var srcPath = _path + selection[1] /*itemname*/;
    
    var pathDisply = srcPath + "' to: ";
    if (srcPath.length > 35) {
        pathDisply = "'/..." + srcPath.substring(srcPath.length - 35, srcPath.length) + "' to: ";
    }
    
    Ext.Msg.prompt('Move or Rename', pathDisply, function(button, text) {       
        if (button == "ok") {
            var destPath = text;
            if (destPath != null && destPath != "" && srcPath !== destPath) {
                var sftpreq = {};
                sftpreq.type = "lmv";
                sftpreq.path = srcPath;
                sftpreq.newpath = destPath;
                session.sendRequest(sftpreq);
            }
        }
    },
    null,
    false,
    srcPath);
};

this.createDirectory = function()
{
    Ext.Msg.prompt('Mkdir', 'Please enter directory name:', function(button, text) {
        if (button == "ok") {
            var dirname = text;
            if (dirname != null && dirname != "") {
                var sftpreq = {};
                sftpreq.type = "lmd";
                sftpreq.path = _path + "/" + dirname;
                session.sendRequest(sftpreq);
            }
        }
    });
};

function updateView(data) {
    var rows = Array();
    for (i = 0; i < data.length; i++) {
        var item = data[i];
        if ("." === item.name) {
            continue;
        }

        var strTime = SFTP.Utility.dateFormat(item.mtime);
        var strSize = '';
        var icon = "";
        
        if (".." === item.name) {
            icon = '<img border="0" align="left" style="padding-right: 3px" src="resources/images/sftp_upfolder.gif"/>';
            item.type = 999;
        } else if (item.type === 2) {
            icon = '<img border="0" align="left" style="padding-right: 3px" src="resources/images/sftp_folder.gif"/>';
        } else {
            var iconFile = SFTP.Utility.getIconForFile(item.name.split('.').pop());
            icon = '<img border="0" align="left" style="padding-right: 3px" src="resources/images/' + iconFile + '"/>';
            strSize = SFTP.Utility.addCommas(item.size);
        }
        rows.push({
            cell: [
                i,
                icon + item.name,
                strSize,
                strTime,
                item.type
            ]
        });
    }

    $(gridControl).flexAddData({
        total: rows.length,
        page: 1,
        rows: rows
    });
    
    // simulate a click on the header to triger the default sorting
    $(gridControl).parent().parent().find("th[abbr='"+'name'+"']").addClass('thOver');
    $(gridControl).parent().parent().find("th[abbr='"+'name'+"']").trigger("click", false /*disable change order*/);
    _path = _newpath;
	
    $(pathControl).val(_path);
    
    onUpdateState('updateviewlocal');	
};

handleResponse = function(rsp) {
    if (rsp.subtype === "lls") {
        document.body.style.cursor = 'default';
        if (0 === rsp.status) {
            updateView(rsp.data);
        } else {
            onUpdateState('commandcompletelocal', rsp.data);

            Ext.Msg.alert('File Transfer - ' + rsp.subtype, rsp.data);
        }
    }
    else if (rsp.subtype === "lmv" ||
             rsp.subtype === "ldf" ||
             rsp.subtype === "lmd" ||
             rsp.subtype === "lrd") {
        if (0 === rsp.status) {
            _that.listDirectory(_path);
        } else {
            onUpdateState('commandcompletelocal', rsp.data);

            document.body.style.cursor = 'default';
            Ext.Msg.alert('File Transfer - ' + rsp.subtype, rsp.data);
        }
    }
}

init();

};
